import config as cf

import networkx as nx


def network_f(distance,time,n_ins):
    '''
    Calculate the last time period that a vehicle can pick up
    evacuees from each pickup zone in order to reach the highway
    by time T+1.
    '''
    t_final = {zone: cf.T + 1 - cf.h_time[zone] for zone in cf.pickup[n_ins]}
    
    '''
    Create a spatiotemporal subnetwork for each vehicle v induced 
    by the set of nodes that are reachable from vehicle origin O_v.
    '''
    
    # emergency vehicle origin subnetworks
    G_r = {''.join(['Or',str(i)]): 
            subgraph_reachable_from_origin(nx.DiGraph(), i, 'or',
                                           n_ins, distance, time, t_final) 
            for i in cf.r_origins[n_ins]}
        
    r_origin_names = G_r.keys()
    
    # volunteer vehicle origin subnetworks
    G_s = {''.join(['Os',str(i)]): 
            subgraph_reachable_from_origin(nx.DiGraph(), i, 'os',
                                           n_ins, distance, time, t_final) 
            for i in cf.s_origins[n_ins]}
    
    s_origin_names = G_s.keys()
    
    G_o = {}
    G_o.update(G_r)
    G_o.update(G_s)
    
    G = nx.compose_all(list(G_o.values()))
    
    p_nodes = [n for n, nodetype in G.nodes(data='nodetype') if nodetype=='p']
    
    origins = [name for name in r_origin_names]
    origins.extend([name for name in s_origin_names])
    
    
    return (origins, G_o, p_nodes, t_final)

'''
Create a subnetwork of nodes reachable from an origin node
used to define flow constraints for each car
'''
def subgraph_reachable_from_origin(G, origin_num, ntyp, n_ins, distance, time, 
                                   t_final):
    h_time = cf.h_time
    h_dist = cf.h_dist
    TT = cf.T + 1
    
    start = {x: int(time.iat[origin_num,x]) for x in cf.pickup[n_ins]}
    
    start[origin_num] = 1 # the pickup location is physically in the same 
                          # place but begin in period 1
    
    # create origin and first destinations
    if ntyp == 'or':
        origin = ''.join(['Or',str(origin_num)])
    elif ntyp == 'os':
        origin = ''.join(['Os',str(origin_num)])
    else:
        print("The subgraph only should make networks with origin of type"
              +" or and type os.")
    
    G.add_node((origin,0), nodetype = ntyp, tme=0, plce=origin)
    
    '''
    Restrict pickup zones to those for which a vehicle can leave the origin,
    arrive, and reach highway before TT
    '''
    
    n_h_pickup = [zone for zone in cf.pickup[n_ins] 
                  if start[zone] <= t_final[zone]]
    
    # create highway nodes
    G.add_node(('h',TT), nodetype='h', tme=TT, plce='h')


    # create pickup nodes 
    for i in n_h_pickup:
        for t in range(start[i], t_final[i]+1):
            G.add_node((i,t), nodetype='p', tme=t, plce=str(i))
    
    '''
    Create idle arcs, travel arcs, highway arcs. 
    "Move" tracks whether the vehicle ever leaves the origin zone
    (besides at the end of the epoch to reach the highway).      
    '''
    
    for i in n_h_pickup:
        if i != origin_num:
            G.add_edge( (origin,0), (i,start[i]), 
                       dist=distance.iat[origin_num,i], move=True)
        else:
            G.add_edge((origin,0), (i,start[i]), 
                       dist=cf.distOrigin_to_Pickup_same_zone, move=False)
        for t in range(start[i],t_final[i]):
            G.add_edge((i,t), (i,t+1), dist=0, move=False)
            for j in n_h_pickup:
                if j != i and time.iat[i,j] + h_time[j] <= TT-t:
                    G.add_edge((i,t), (j,t+int(time.iat[i,j])), 
                               dist=distance.iat[i,j], move=True)
        G.add_edge((i,t_final[i]), ('h',TT), dist=h_dist[i], move = False)
    G.add_edge(('h',TT),(origin,0),dist=0, move = False)

    return(G)
 